'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _singleSpaReact = require('./single-spa-react.js');

var _singleSpaReact2 = _interopRequireDefault(_singleSpaReact);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('single-spa-react', function () {
  var React = void 0,
      ReactDOM = void 0,
      rootComponent = void 0,
      domElement = void 0,
      domElementGetter = void 0,
      componentInstance = void 0,
      createdReactElement = void 0;

  beforeEach(function () {
    React = {
      createElement: jest.fn(function () {
        return createdReactElement;
      }),
      version: '16.2.0'
    }, ReactDOM = {
      render: jest.fn(function (reactEl, domEl, cbk) {
        cbk();
        return componentInstance;
      }),
      unmountComponentAtNode: jest.fn()
    };

    createdReactElement = "Hey a created react element";
    componentInstance = { componentDidCatch: function componentDidCatch() {} };
    rootComponent = jest.fn();
    domElement = "Hey i'm the dom element";
    domElementGetter = function domElementGetter() {
      return domElement;
    };

    console.warn = jest.fn();
  });

  it('throws an error if you don\'t pass required opts', function () {
    expect(function () {
      return (0, _singleSpaReact2.default)();
    }).toThrow();
    expect(function () {
      return (0, _singleSpaReact2.default)({});
    }).toThrow();
    expect(function () {
      return (0, _singleSpaReact2.default)({ ReactDOM: ReactDOM, rootComponent: rootComponent });
    }).toThrow();
    expect(function () {
      return (0, _singleSpaReact2.default)({ React: React, rootComponent: rootComponent });
    }).toThrow();
    expect(function () {
      return (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM });
    }).toThrow();
  });

  it('mounts and unmounts a React component, passing through the single-spa props', function () {
    var props = { why: 'hello', customProps: {} };
    var lifecycles = (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM, rootComponent: rootComponent, domElementGetter: domElementGetter });

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    }).then(function () {
      expect(React.createElement).toHaveBeenCalled();
      expect(React.createElement.mock.calls[0][0]).toEqual(rootComponent);
      expect(React.createElement.mock.calls[0][1]).toEqual(props);
      expect(ReactDOM.render).toHaveBeenCalled();
      expect(ReactDOM.render.mock.calls[0][0]).toEqual(createdReactElement);
      expect(ReactDOM.render.mock.calls[0][1]).toEqual(domElement);
      expect(_typeof(ReactDOM.render.mock.calls[0][2])).toEqual('function');
      return lifecycles.unmount(props);
    }).then(function () {
      expect(ReactDOM.unmountComponentAtNode).toHaveBeenCalledWith(domElement);
    });
  });

  it('chooses the parcel dom element over other dom element getters', function () {
    var optsDomElementGetter = function optsDomElementGetter() {
      return 'optsDomElementGetter';
    };
    var opts = { React: React, ReactDOM: ReactDOM, rootComponent: rootComponent, domElementGetter: optsDomElementGetter };
    var propsDomElementGetter = function propsDomElementGetter() {
      return 'propsDomElementGetter';
    };
    var propsDomElement = function propsDomElement() {
      return 'propsDomElement';
    };
    var props = { customProps: { domElement: propsDomElement, domElementGetter: propsDomElementGetter } };

    var lifecycles = (0, _singleSpaReact2.default)(opts);

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    }).then(function () {
      return lifecycles.unmount(props);
    }).then(function () {
      expect(ReactDOM.render).toHaveBeenCalled();
      // prefer customProp dom element over everything because it's how parcels work
      expect(ReactDOM.render.mock.calls[0][1]).toBe(propsDomElement);
    });
  });

  it('allows you to provide a domElementGetter as an opt', function () {
    var props = { why: 'hello', customProps: {} };
    var lifecycles = (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM, rootComponent: rootComponent, domElementGetter: domElementGetter });

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    });
    // Doesn't throw
  });

  it('allows you to provide a domElementGetter as a customProps', function () {
    var props = { why: 'hello', customProps: { domElementGetter: domElementGetter } };
    var lifecycles = (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM, rootComponent: rootComponent });

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    });
    // Doesn't throw
  });

  it('throws if you provide no way of getting a dom element', function () {
    var opts = { React: React, ReactDOM: ReactDOM, rootComponent: rootComponent };
    var props = { customProps: {} };

    var lifecycles = (0, _singleSpaReact2.default)(opts);

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    }).then(function () {
      return Promise.reject('expected error because no dom element');
    }, function () {
      return Promise.resolve('error expected because no dom element');
    });
  });

  it('warns if you are using react 16 but don\'t implement componentDidCatch', function () {
    delete componentInstance.componentDidCatch;
    React.version = '16.2.0';
    var props = { why: 'hello', customProps: {} };
    var lifecycles = (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM, rootComponent: rootComponent, domElementGetter: domElementGetter });

    return lifecycles.bootstrap().then(function () {
      return expect(console.warn).not.toHaveBeenCalled();
    }).then(function () {
      return lifecycles.mount(props);
    }).then(function () {
      return expect(console.warn).toHaveBeenCalled();
    });
  });

  it('does not warn if you are using react 15 but don\'t implement componentDidCatch', function () {
    delete componentInstance.componentDidCatch;
    React.version = '15.4.1';
    var props = { why: 'hello', customProps: {} };
    var lifecycles = (0, _singleSpaReact2.default)({ React: React, ReactDOM: ReactDOM, rootComponent: rootComponent, domElementGetter: domElementGetter });

    return lifecycles.bootstrap().then(function () {
      return lifecycles.mount(props);
    }).then(function () {
      return expect(console.warn).not.toHaveBeenCalled();
    });
  });
});
//# sourceMappingURL=single-spa-react.test.js.map